package com.huigou.topsun.base.bsnMessage.appliction.impl;

import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.base.bsnMessage.appliction.BsnMessageApplication;
import com.huigou.topsun.base.bsnMessage.domain.*;
import com.huigou.topsun.base.bsnMessage.repository.BsnMessageExecuteRepository;
import com.huigou.topsun.base.bsnMessage.repository.BsnMessageRejectRepository;
import com.huigou.topsun.base.bsnMessage.repository.BsnMessageSponsorRepository;
import com.huigou.uasp.bmp.common.application.BaseApplication;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Sort;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 专项任务整合及输出
 */
@Service("bsnMessageApplication")
public class BsnMessageApplicationImpl extends BaseApplication implements BsnMessageApplication {

    @Autowired
    BsnMessageExecuteRepository messageExecuteRepository;
    @Autowired
    BsnMessageRejectRepository messageRejectRepository;
    @Autowired
    BsnMessageSponsorRepository messageSponsorRepository;


    @Override
    public Map<String, Object> queryExecuteBsnMessage() {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "queryExecuteBsnMessage");
        QueryModel queryModel = new QueryModel();
        queryModel.setSql(queryDescriptor.getSql());
        queryModel.setPageIndex(1);
        queryModel.setPageSize(10);
        queryModel.setSortFieldName("updateDate");
        queryModel.setSortOrder("desc");
        queryModel.putParam("taskExecute",this.getOperator().getUserId());
        queryModel.putParam("addText", "[驳回]");
        return this.sqlExecutorDao.executeSlicedQuery(queryModel);
    }

    @Override
    public List<BsnMessageSponsor> queryFinishBsnMessageSponsor() {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "queryFinishBsnMessageSponsor");
        return this.sqlExecutorDao.queryToList(queryDescriptor.getSql(), BsnMessageSponsor.class,this.getOperator().getUserId());
    }



    public void saveBsnMessageSponsor(List<BsnMessageSponsor> all){
        messageSponsorRepository.save(all);
    }

    public String saveBsnMessageSponsor(BsnMessageSponsor entity){
        String keyId = messageSponsorRepository.save(entity).getId();
        if(isNotEmpty(entity.getExecutes())){
            entity.getExecutes().forEach(item->item.setMessageSponsorId(keyId));
            messageExecuteRepository.save(entity.getExecutes());
        }
        return keyId;
    }


    public void saveBsnMessageExecute(List<BsnMessageExecute> all){
        messageExecuteRepository.save(all);
    }
    public String saveBsnMessageExecute(BsnMessageExecute entity){
        String keyId = messageExecuteRepository.save(entity).getId();
        if(isNotEmpty(entity.getRejects())){
            entity.getRejects().forEach(item->item.setBsnExecuteId(keyId));
            messageRejectRepository.save(entity.getRejects());
        }
        return keyId;
    }
    public void saveBsnMessageReject(List<BsnMessageReject> all){
        messageRejectRepository.save(all);
    }
    public void saveBsnMessageReject(BsnMessageReject entity){
        messageRejectRepository.save(entity);
    }

    @Override
    public void deleteBsnMessageSponsor(BsnMessageSponsor entity) {
        if(isNotEmpty(entity.getExecutes())){
            messageExecuteRepository.delete(entity.getExecutes());
        }
        messageSponsorRepository.delete(entity);
    }

    @Override
    public List<BsnMessageExecute> getExecuteByMsId(String msId) {
        return messageExecuteRepository.findAllByMessageSponsorId(msId);
    }

    @Override
    public BsnMessageExecute getExecuteByMeId(String id) {
        return messageExecuteRepository.findOne(id);
    }

    @Override
    public List<BsnMessageReject> getRejectByMeId(String meId) {
        Sort sort = new Sort(Sort.Direction.DESC, "rejectDate");
        return messageRejectRepository.findAllByBsnExecuteId(meId,sort);
    }

    @Override
    public List<BsnMessageExecute> getBrotherExecuteByMeId(String meId) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "queryBrotherExecute");
        Map<String,Object> mapParam = new HashMap<>();
        mapParam.put("meId",meId);
        return this.sqlExecutorDao.queryToListByMapParam(queryDescriptor.getSql(), BsnMessageExecute.class, mapParam);
    }

    @Override
    public BsnMessageSponsor getSponsorByBId(String bId) {
        BsnMessageSponsor sponsor = messageSponsorRepository.findByBusinessId(bId);
        if(sponsor!=null){
            List<BsnMessageExecute> executeList =this.getExecuteByMsId(sponsor.getId());
            sponsor.setExecutes(executeList);
        }
        return sponsor;
    }

    public boolean isNotEmpty(List<?> list) {
        return list != null && !list.isEmpty();
    }
}
